@@ -64,15 +64,13 @@ int main(int argc, char **argv) {
         singularity_registry_set("IMAGE", argv[1]);
     }
 
-    singularity_sessiondir();
-
     image = singularity_image_init(singularity_registry_get("IMAGE"));
 
-    singularity_image_open(&image, O_RDWR);
+//    singularity_image_open(&image, O_RDWR);
+//
+//    singularity_image_check(&image);
 
-    singularity_image_check(&image);
-
-    if ( image.type != SINGULARITY ) {
+    if ( image.type != EXT3 ) {
         singularity_message(ERROR, "Import is only allowed on Singularity image files\n");
         ABORT(255);
     }
@@ -81,7 +79,7 @@ int main(int argc, char **argv) {
 
     singularity_runtime_ns(SR_NS_MNT);
 
-    singularity_image_bind(&image);
+//    singularity_image_bind(&image);
 
     if ( image.loopdev == NULL ) {
         singularity_message(ERROR, "Bind failed to connect to image!\n");
@@ -92,9 +90,9 @@ int main(int argc, char **argv) {
 
     // Check to make sure the image hasn't been swapped out by a race
     image_test = singularity_image_init(singularity_registry_get("IMAGE"));
-    singularity_image_open(&image_test, O_RDONLY);
-    singularity_image_check(&image_test);
-    if ( image_test.type != SINGULARITY ) {
+//    singularity_image_open(&image_test, O_RDONLY);
+//    singularity_image_check(&image_test);
+    if ( image_test.type != EXT3 ) {
         singularity_message(ERROR, "Import is only allowed on Singularity image files\n");
         ABORT(255);
     }
@@ -126,7 +124,7 @@ int main(int argc, char **argv) {
 
     singularity_priv_escalate();
     singularity_message(VERBOSE, "Opening STDIN for tar stream\n");
-    retval = singularity_fork_exec(tar_cmd);
+    retval = singularity_fork_exec(0, tar_cmd);
     singularity_priv_drop();
 
     if ( retval != 0 ) {
