@@ -35,19 +35,21 @@
 #include <libgen.h>
 #include <linux/limits.h>
 
+#include "config.h"
 #include "util/file.h"
 #include "util/util.h"
 #include "util/message.h"
 #include "util/privilege.h"
 #include "util/config_parser.h"
 #include "util/registry.h"
+#include "util/mount.h"
 
 #include "../file-bind.h"
 #include "../../runtime.h"
 
 
 int _singularity_runtime_files_libs(void) {
-    char *container_dir = singularity_runtime_rootfs(NULL);
+    char *container_dir = CONTAINER_FINALDIR;
     char *tmpdir = singularity_registry_get("SESSIONDIR");
     char *includelibs_string;
     char *libdir = joinpath(tmpdir, "/libs");
@@ -134,7 +136,7 @@ int _singularity_runtime_files_libs(void) {
 
             singularity_priv_escalate();
             singularity_message(VERBOSE, "Binding file '%s' to '%s'\n", source, dest);
-            if ( mount(source, dest, NULL, MS_BIND|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0 ) {
+            if ( singularity_mount(source, dest, NULL, MS_BIND|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0 ) {
                     singularity_priv_drop();
                     singularity_message(ERROR, "There was an error binding %s to %s: %s\n", source, dest, strerror(errno));
                     ABORT(255);
@@ -167,7 +169,7 @@ int _singularity_runtime_files_libs(void) {
 
         singularity_priv_escalate();
         singularity_message(VERBOSE, "Binding libdir '%s' to '%s'\n", libdir, libdir_contained);
-        if ( mount(libdir, libdir_contained, NULL, MS_BIND|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0 ) {
+        if ( singularity_mount(libdir, libdir_contained, NULL, MS_BIND|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0 ) {
                 singularity_priv_drop();
                 singularity_message(ERROR, "There was an error binding %s to %s: %s\n", libdir, libdir_contained, strerror(errno));
                 ABORT(255);
