@@ -40,11 +40,10 @@
 #include "util/privilege.h"
 #include "util/fork.h"
 #include "util/registry.h"
+#include "util/setns.h"
 
 
 int _singularity_runtime_ns_pid(void) {
-    int enabled = -1;
-
 
 #ifdef SINGULARITY_NO_NEW_PRIVS
     // Use PID namespace when NO_NEW_PRIVS is not supported
@@ -57,53 +56,56 @@ int _singularity_runtime_ns_pid(void) {
         singularity_message(VERBOSE2, "Not virtualizing PID namespace on user request\n");
         return(0);
     }
-#endif
+#endif /* SINGULARITY_NO_NEW_PRIVS */
 
 #ifdef NS_CLONE_NEWPID
     singularity_message(DEBUG, "Using PID namespace: CLONE_NEWPID\n");
-    singularity_priv_escalate();
-    singularity_message(DEBUG, "Virtualizing PID namespace\n");
-    if ( unshare(CLONE_NEWPID) < 0 ) {
-        singularity_message(ERROR, "Could not virtualize PID namespace: %s\n", strerror(errno));
-        ABORT(255);
-    }
-    singularity_priv_drop();
-    enabled = 1;
-
-#else
-#ifdef NS_CLONE_PID
-    singularity_message(DEBUG, "Using PID namespace: CLONE_PID\n");
-    singularity_priv_escalate();
-    singularity_message(DEBUG, "Virtualizing PID namespace\n");
-    if ( unshare(CLONE_NEWPID) < 0 ) {
-        singularity_message(ERROR, "Could not virtualize PID namespace: %s\n", strerror(errno));
-        ABORT(255);
-    }
-    singularity_priv_drop();
-    enabled = 1;
-
+    
 #else
     singularity_message(WARNING, "Skipping PID namespace creation, support not available on host\n");
     return(0);
-
-#endif
 #endif
 
-    if ( enabled == 1 ) {
-        // PID namespace requires a fork to activate!
-        singularity_fork_run();
-
-        singularity_registry_set("PIDNS_ENABLED", "1");
+    singularity_message(DEBUG, "Virtualizing PID namespace\n");
+        
+    if ( singularity_registry_get("DAEMON_START") ) {
+        singularity_fork_daemonize(CLONE_NEWPID);
+    } else {
+        singularity_fork_run(CLONE_NEWPID);
     }
 
+    singularity_registry_set("PIDNS_ENABLED", "1");
+
     return(0);
 }
 
+int _singularity_runtime_ns_pid_join(void) {
+    int ns_fd = atoi(singularity_registry_get("DAEMON_NS_FD"));
+    int pid_fd;
 
+    /* Attempt to open /proc/[PID]/ns/pid */
+    singularity_priv_escalate();
+    pid_fd = openat(ns_fd, "pid", O_RDONLY);
 
-/*
-int singularity_ns_pid_enabled(void) {
-    singularity_message(DEBUG, "Checking PID namespace enabled: %d\n", enabled);
-    return(enabled);
+    if( pid_fd == -1 ) {
+        /* Daemons should always have a ns/pid file. If it doesn't exist, something is wrong */
+        singularity_message(ERROR, "Could not open PID NS fd: %s\n", strerror(errno));
+        ABORT(255);
+    }
+    
+    singularity_message(DEBUG, "Attempting to join PID namespace\n");
+    if ( setns(pid_fd, CLONE_NEWPID) < 0 ) {
+        singularity_message(ERROR, "Could not join PID namespace: %s\n", strerror(errno));
+        ABORT(255);
+    }
+    singularity_priv_drop();
+    singularity_message(DEBUG, "Successfully joined PID namespace\n");
+    
+    close(pid_fd);
+    
+    /* Enable PID NS by forking into a child */
+    singularity_fork_run(0);
+    singularity_registry_set("PIDNS_ENABLED", "1");
+    
+    return(0);
 }
-*/
