@@ -29,16 +29,13 @@ test_init "Checking escalation block"
 
 CONTAINER="$SINGULARITY_TESTDIR/container.img"
 
-stest 0 singularity create -s 568 "$CONTAINER"
-stest 0 sudo singularity bootstrap "$CONTAINER" docker://centos:7
+stest 0 sudo singularity build "$CONTAINER" docker://centos:7
 stest 0 singularity exec "$CONTAINER" true
 stest 1 singularity exec "$CONTAINER" false
 
 # Checking no new privs with capabilities
-stest 0 sudo singularity exec "$CONTAINER" ping localhost -c 1
-stest 1 singularity exec "$CONTAINER" ping localhost -c 1
-stest 0 sudo singularity exec "$CONTAINER" ping localhost -c 1
-stest 1 singularity exec "$CONTAINER" ping localhost -c 1
+stest 0 sudo singularity exec "$CONTAINER" chsh -s /bin/sh
+stest 1 singularity exec "$CONTAINER" chsh -s /bin/sh
 
 
 test_cleanup
